# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright (C) 2014 Xavier Bruhiere


import time
import os
from flask.ext import restful
from flask.ext.restful import reqparse
from redis import Redis

from telepathy.portfolios import RethinkdbPortfolios
from telepathy.trades import trade


class Trade(restful.Resource):
    jobs = {}

    def get(self, portfolio_id):
        ''' Return status report '''

        if portfolio_id in self.jobs:
            elapsed = time.time() - self.jobs[portfolio_id]['start']
            worker = self.jobs[portfolio_id]['worker']

            code = 200
            report = {
                'elapsed': elapsed,
                'id': portfolio_id,
                'done': worker.ready(),
                'task_id': worker.id,
                'task': worker.task_name,
                'status': worker.status,
                'state': worker.state}
            if worker.ready():
                report.update({
                    'successful': worker.successful(),
                    'failed': worker.failed()})
                if worker.successful():
                    report['result'] = worker.get()
                else:
                    report['traceback'] = worker.traceback
                    report['result'] = str(worker.get(propagate=False))
                    code = 500

        else:
            report = {'error': 'job {} unknown'.format(portfolio_id)}
            code = 404

        return report, code

    def post(self, portfolio_id):
        parser = reqparse.RequestParser()
        parser.add_argument('context', type=str)
        parser.add_argument('bot', type=bool)
        args = parser.parse_args()

        #TODO ctx = args.get('context', node._detect_context())
        worker = trade.delay(
            portfolio_id, args['context'], args.get('bot', False))

        self.jobs[portfolio_id] = {
            'worker': worker,
            'start': time.time()}

        return {'enqueued': True,
                'id': portfolio_id,
                'task_id': worker.task_id,
                'start': time.time()}

    def delete(self, portfolio_id):
        ''' Stop the trade <portfolio_id> '''
        if portfolio_id in self.jobs:
            self.jobs[portfolio_id]['worker'].revoke(terminate=True)
            report = {}
        else:
            report = {'error': 'job {} unknown'.format(portfolio_id)}
        return report


class Portfolios(restful.Resource):
    db = RethinkdbPortfolios()

    def get(self):
        parser = reqparse.RequestParser()
        parser.add_argument('id', type=str, action='append', required=True)
        parser.add_argument('key', type=str, action='append')
        parser.add_argument('from', type=int)
        parser.add_argument('to', type=int)
        #TODO parser.add_argument('date', type=str)
        #TODO parser.add_argument('less', type=float)
        #TODO parser.add_argument('more', type=float)
        args = parser.parse_args()

        try:
            if args['from']:
                data = self.db.build_datapoints(
                    args['id'], args['key'], args['from'], args['to'])
            else:
                data = self.db.build_values(args['id'], args['key'])
            code = 200
        except Exception, error:
            data = {'error': repr(error)}
            code = 404

        return data, code


class Order(restful.Resource):

    conn = Redis(
        host=os.getenv('BROKER_HOST', 'localhost'),
        port=int(os.getenv('BROKER_PORT', 6379)))

    def post(self, portfolio_id):
        parser = reqparse.RequestParser()
        parser.add_argument('deal', type=str, action='append', required=True)
        args = parser.parse_args()

        data = {}
        for deal in args['deal']:
            sid = deal.split(':')[0]
            amount = deal.split(':')[1]
            data[sid] = amount

        try:
            self.conn.rpush(portfolio_id, data)
            report = {'processed': True, 'transaction': data}
            code = 200
        except Exception, error:
            report = {'error': error}
            code = 500

        return report, code


class Doc(restful.Resource):

    def get(self):
        return {
            'telepathy': 'version 0.0.4',
            'endpoints': {
                '/api/v1/doc': Doc.__doc__,
                '/api/v1/portfolios': Portfolios.__doc__,
                '/api/v1/trades/:id': Trade.__doc__,
                '/api/v1/order/:id': Order.__doc__}}
